# This file contains all constants which are used throughout the program

import os
import play_state

# DEBUG and RELEASE are used in the code for special conditions that should
# only be used in the corresponding state. This enables us to take out all
# debug functionality with a single change here.
DEBUG = False
RELEASE = not DEBUG

SKIN_REALISTIC = "realistic"
SKIN_TRON = "tron"
SHARED_SKIN_FOLDER = "shared"

DEBUG_SKIP_MENU = True and DEBUG
DEBUG_SKIP_MENU_DIFFICULTY = play_state.PlayState.DIFFICULTY_EASY
DEBUG_SKIP_MENU_DURATION = play_state.PlayState.DURATION_LONG
DEBUG_SKIP_MENU_SKIN = SKIN_TRON
DEBUG_BOX2D_OUTPUT = False and DEBUG

WINDOW_WIDTH = 800
WINDOW_HEIGHT = 600

CONFIG_FILE = "settings.cfg"

RESOURCES_DIRECTORY = "resources"
IMAGES_DIRECTORY = os.path.join(RESOURCES_DIRECTORY, "images")
SOUND_DIRECTORY = os.path.join(RESOURCES_DIRECTORY, "sounds")
MUSIC_DIRECTORY = os.path.join(RESOURCES_DIRECTORY, "music")
FONT_DIRECTORY = os.path.join(RESOURCES_DIRECTORY, "fonts")

MUSIC_ON_AT_START = True if RELEASE else False
MUSIC_VOLUME_FULL = 0.3

JOYSTICK_CHECK_COUNTDOWN = 2
DESIRED_JOYSTICKS_COUNT = 2
JOYSTICK_DEAD_ZONE = 0.2
JOYSTICK_ACCEPT_ZONE = 0.6

WORLD_TO_SCREEN = 100
SCREEN_TO_WORLD = 1.0 / WORLD_TO_SCREEN

FRAME_RATE = 60

PHYSICS_SPEED = 0.3

GRAVITY_ENABLED = False
GRAVITY = -5 if GRAVITY_ENABLED else 0
PLAYER_GRAVITY_SCALE = 0
HAMMER_GRAVITY_SCALE = 1
BALL_GRAVITY_SCALE = 0.5

PLAYER_DENSITY = 1
PLAYER_FORCE = 8
PLAYER_CAN_HIT_BALL = False
PLAYER_CAN_HIT_ENEMY_COLORED_BALL = True
PLAYER_FRICTION = 0.2

PLAYER_HAMMER_MAX_DISTANCE_WORLD = 150 * SCREEN_TO_WORLD
PLAYER_HAMMER_INITIAL_DISTANCE_WORLD = 100 * SCREEN_TO_WORLD
PLAYER_HAMMER_STIFFNESS = 40

HAMMERS_COLLIDE = True
HAMMERS_COLLIDE_WITH_BORDER = False
HAMMER_LINEAR_DAMPING = 2
HAMMER_DENSITY = 1
HAMMER_FRICTION = 0.2

HAMMER_HAMMER_COLLISON_PARTICLE_OFFSET_DISTANCE = 0.02

BALL_LINEAR_DAMPING = 1
BALL_DENSITY = 1
BALL_FRICTION = 0.2
BALL_RESTITUTION = 0.7

BALL_THROW_IN_COUNTDOWN_START = 5
BALL_THROW_IN_COUNTDOWN = 2
BALL_THROW_IN_MIN_SPEED = 10
BALL_THROW_IN_MAX_SPEED = 70

BALL_TRAIL_DURATION = 1
BALL_TRAIL_FADE_OUT_DURATION = BALL_TRAIL_DURATION

USER_DATA_ENTITY = 1
USER_DATA_GOAL = 2

FILTER_BORDER = 1
FILTER_HAMMER = 2
FILTER_BALL = 4
FILTER_PLAYER = 8
FILTER_PLAYER_1 = 16
FILTER_PLAYER_2 = 32

SCREEN_SHAKE_DELAY_BETWEEN_SHAKES = 0.02
SCREEN_SHAKE_BALL_HAMMER_DISTANCE = 2.8
SCREEN_SHAKE_BALL_HAMMER_DURATION = 0.5
SCREEN_SHAKE_BALL_BORDER_DISTANCE = 1.1
SCREEN_SHAKE_BALL_BORDER_DURATION = 0.5
#SCREEN_SHAKE_BALL_PLAYER_DISTANCE = 0
#SCREEN_SHAKE_BALL_PLAYER_DURATION = 0
SCREEN_SHAKE_HAMMER_HAMMER_DISTANCE = 5.6 
SCREEN_SHAKE_HAMMER_HAMMER_DURATION = 0.5

SOUND_GAME_STARTS = "game_starts"
SOUND_BALL_THROWN_IN = "ball_thrown_in"
SOUND_COLLISION_BALL_PLAYER = "collision_ball_player"
SOUND_COLLISION_BALL_HAMMER = "collision_ball_hammer"
SOUND_COLLISION_BALL_BORDER = "collision_ball_border"
SOUND_COLLISION_HAMMER_HAMMER = "collision_hammer_hammer"
SOUND_GOAL = "goal"
SOUND_OWN_GOAL = "own_goal"
SOUND_GAME_OVER = "game_over"
SOUND_PAUSE = "pause"
SOUND_RESUME = "resume"
SOUND_MENU_MOVED = "menu_moved"
SOUND_MENU_SELECTION = "menu_selection"
SOUND_MENU_ACTION = "menu_action"
SOUND_MENU_DENIED = "menu_denied"

SOUNDS = [
          SOUND_GAME_STARTS,
          SOUND_BALL_THROWN_IN,
          SOUND_COLLISION_BALL_PLAYER,
          SOUND_COLLISION_BALL_HAMMER,
          SOUND_COLLISION_BALL_BORDER,
          SOUND_COLLISION_HAMMER_HAMMER,
          SOUND_GOAL,
          SOUND_OWN_GOAL,
          SOUND_GAME_OVER,
          SOUND_PAUSE,
          SOUND_RESUME,
          SOUND_MENU_MOVED,
          SOUND_MENU_SELECTION,
          SOUND_MENU_ACTION,
          SOUND_MENU_DENIED
          ]
