import math

# Simple point class adding some functionality absent from 
# the tuples normally used throughout pygame
class Point:
	def __init__(self, x=0, y=0):
		self.x = x
		self.y = y
		
	def as_tuple(self):
		return (self.x, self.y)
	
	def set_to(self, x, y):
		self.x = x
		self.y = y
		
	def move(self, dx, dy):
		self.x += dx
		self.y += dy
		
	def length(self):
		return math.sqrt(self.x * self.x + self.y * self.y)
	
	def normalize(self):
		length = self.length()
		
		if length == 0:
			return
		
		self.x /= length
		self.y /= length
		return length
	
	def multiply(self, value):
		self.x *= value
		self.y *= value
	
	def add(self, other_point):
		return Point(self.x + other_point.x, self.y + other_point.y)
	
	def subtract(self, other_point):
		return Point(self.x - other_point.x, self.y - other_point.y)
	
	def angle(self):
		return math.atan2(self.y, self.x)
	
	def distance(self, other_point):
		dx = self.x - other_point.x
		dy = self.y - other_point.y
		return math.sqrt(dx * dx + dy * dy)