import constants
import pygame
import data
import utils
import math


class State(object):
    
    def __init__(self):
        pass
    
    def init(self):
        pass
    
    def destroy(self):
        pass

    def update(self):
        pass
    
    def draw(self, surface):
        pass
    
    def handle_event(self):
        pass

class EntityGroup(pygame.sprite.Group):
    
    def _init_(self):
        pygame.sprite.Group.__init__(self)
        
    def draw(self, surface):
        entities = self.sprites()
        for entity in entities:
            entity.draw(surface);


# Base entity class, providing basic positional functionality and an event handler
class Entity(pygame.sprite.Sprite):

    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.origin = data.Point()                  # Origin of the drawn image: Here is (0|0) 
        self.position = data.Point()                # Position of the image origin on the screen
        self.image = 0                              # Initialize image used by pygame.Group to draw
        self.rect = pygame.rect.Rect(0, 0, 0, 0)    # Initialize image rectangle used by pygame.Group to draw
        self.body = None
        self.texture = None

    def set_image(self, image_and_rect_tuple):
        self.image, self.rect = image_and_rect_tuple
        self.texture = self.image

    def set_position(self, x, y):
        self.position.set_to(x, y)
        self.update_rect()
        
    def move(self, offset_x, offset_y):
        self.position.move(offset_x, offset_y)
        self.update_rect()
        
    def apply_body_to_entity(self, apply_rotation):
        if self.body == None:
            return

        converted_position = utils.world_to_screen(self.body.position.x, self.body.position.y)
        self.position.set_to(converted_position[0], converted_position[1])

        if apply_rotation:
            self.set_rotation(math.degrees(self.body.angle))

        self.rect = self.image.get_rect()
        self.update_rect();
        
    def set_rotation(self, angle):
        if angle == 0.0:
            self.image = self.texture
        else:
            self.image = pygame.transform.rotate(self.texture, angle)

        #self.rect = self.image.get_rect()
        
    def update_rect(self):
        # Update the rectangle used by pygame.Group to draw when the position was updated
        if self.body == None:
            self.rect.left = self.position.x - self.origin.x 
            self.rect.top = self.position.y - self.origin.y
        else:
            self.rect.left = self.position.x - self.rect.width / 2
            self.rect.top = self.position.y - self.rect.height / 2
        return
    
    # Limits the entity to the current window, correcting it's position and returning
    # True when correction was necessary, else false
    def limit_to_window(self):
        hit_border = False
        
        if self.position.x < 0:
            self.position.x = 0
            hit_border = True
        
        if self.position.x >= constants.WINDOW_WIDTH:
            self.position.x = constants.WINDOW_WIDTH-1
            hit_border = True
            
        if self.position.y < 0:
            self.position.y = 0
            hit_border = True
        
        if self.position.y >= constants.WINDOW_HEIGHT:
            self.position.y = constants.WINDOW_HEIGHT-1
            hit_border = True
            
        if hit_border:
            self.update_rect()
            
        return hit_border

    # Set the origin to the center of the image
    def center(self):
        self.origin.set_to(self.rect.width / 2, self.rect.height / 2)
        self.update_rect()

    def draw(self, surface):
        surface.blit(self.image, self.rect)

    # Handle an event; to be overridden by subclasses
    def handle_event(self, event):
        return


class Text(Entity):

    H_ALIGN_LEFT = 0
    H_ALIGN_CENTER = 1
    H_ALIGN_RIGHT = 2
    V_ALIGN_TOP = 0
    V_ALIGN_CENTER = 1
    V_ALIGN_BOTTOM = 2
    
    def __init__(self, text, font, position, color=(0, 0, 0), h_align=H_ALIGN_LEFT, v_align=V_ALIGN_TOP):
        Entity.__init__(self)
        self.text_value = None
        self.font = font
        self.position = position
        self.color = color
        self.h_align = h_align
        self.v_align = v_align
        
        self.text = text

    @property
    def color(self):
        return self.color_value

    @color.setter
    def color(self, value):
        self.color_value = value
        self.render_text(self.text_value)
        
    @property
    def font(self):
        return self.font_value

    @font.setter
    def font(self, value):
        self.font_value = value
        self.render_text(self.text_value)
        
    @property
    def text(self):
        return self.text_value

    @text.setter
    def text(self, value):
        self.text_value = value
        self.render_text(self.text_value)
        
    def render_text(self, text):
        if text == None:
            return
        
        self.image = self.font.render(text, True, self.color)
        self.rect = self.image.get_rect()

        self.origin.x = 0
        self.origin.y = 0
        
        if self.h_align == Text.H_ALIGN_CENTER:
            self.origin.x = self.rect.width / 2
        elif self.h_align == Text.H_ALIGN_RIGHT:
            self.origin.x = self.rect.width
        
        if self.v_align == Text.V_ALIGN_CENTER:
            self.origin.y = self.rect.height / 2
        elif self.v_align == Text.V_ALIGN_BOTTOM:
            self.origin.y = self.rect.height
        
        self.update_rect()
