import os
import pygame
import constants
import Box2D
import math


# Load image from correct path and return converted image object
def load_png(name, skin):
    fullname = os.path.join(constants.IMAGES_DIRECTORY, skin, name)
    try:
        image = pygame.image.load(fullname)
        if image.get_alpha() is None:
            image = image.convert()
        else:
            image = image.convert_alpha()
    except pygame.error, message:
            print 'Cannot load image:', fullname
            raise SystemExit, message
    return image, image.get_rect()

# Load music file from the correct path
def load_music(name):
    return pygame.mixer.Sound(os.path.join(constants.MUSIC_DIRECTORY, name))

def load_sound(name, skin):
    path = os.path.join(constants.SOUND_DIRECTORY, skin, name)
    if not os.path.exists(path):
        return None
    
    return pygame.mixer.Sound(path)

def load_font(name, size):
    return pygame.font.Font(os.path.join(constants.FONT_DIRECTORY, name), size)

def screen_to_world(x, y):
    return (x * constants.SCREEN_TO_WORLD,
            (constants.WINDOW_HEIGHT - y) * constants.SCREEN_TO_WORLD)

def world_to_screen(x, y):
    return (x * constants.WORLD_TO_SCREEN,
            constants.WINDOW_HEIGHT - y * constants.WORLD_TO_SCREEN)
    
def create_static_rectangle_body(world, x, y, width, height, category_bits = None, mask_bits = None, user_data=None):
    body_def = Box2D.b2BodyDef()
    body_def.position = screen_to_world(x + width / 2, y + height / 2)
    body = world.CreateBody(body_def)
    
    shape = Box2D.b2PolygonShape(box=(width * constants.SCREEN_TO_WORLD / 2, height * constants.SCREEN_TO_WORLD / 2))
    
    fixture = Box2D.b2FixtureDef(shape=shape, categoryBits=category_bits, maskBits=mask_bits, userData=user_data)
    body.CreateFixture(fixture)

    return body, fixture

def clamp(value, min_value, max_value):
    return max(min(value, max_value), min_value)

def wrap_around_modulo(value, max_value):
    value %= max_value
    
    while value < 0:
        value += max_value
        
    return value

# http://code.activestate.com/recipes/66472-frange-a-range-function-with-float-increments/
def frange(limit1, limit2 = None, increment = 1.):
    """
    Range function that accepts floats (and integers).
    
    Usage:
    frange(-2, 2, 0.1)
    frange(10)
    frange(10, increment = 0.5)
    
    The returned value is an iterator.  Use list(frange) for a list.
    """
    
    if limit2 is None:
        limit2, limit1 = limit1, 0.
    else:
        limit1 = float(limit1)
    
    count = int(math.ceil(limit2 - limit1)/increment)
    return (limit1 + n*increment for n in range(count))
